import { escapeHTMLAttribute, escapeHTMLText } from "../shared/escape.js";
export const EMPTY_ARRAY = [];
const NOOP_1 = (arg1) => { };
export const defineRoot = NOOP_1;
export const context = () => {
    const d = { t: 2 /* VNodeType.Context */, s: null };
    return [
        (component) => _getContextValue(d),
        (v, c) => ({ d, p: { v, c } }),
    ];
};
const _getContextValue = (d) => {
    const ctxStack = RENDER_CONTEXT.c;
    let i = ctxStack.length;
    while (i-- > 0) {
        const c = ctxStack[i];
        if (c.d === d) {
            return c.p.v;
        }
    }
};
export const component = (s, areEqual) => {
    const d = { t: 1 /* VNodeType.Component */, s };
    return (p) => ({ d, p });
};
// SSR passes props value instead of component node
export const getProps = (component) => component;
/** Returns noop function. */
export const useEffect = (component, hook, areEqual) => NOOP_1;
/** Returns noop function. */
export const useLayoutEffect = useEffect;
/** Returns noop function. */
export const useIdleEffect = useEffect;
export const List = (entries, getKey, render) => entries.map(render);
export const _$T = (s) => ({
    t: 0 /* VNodeType.Template */,
    s,
});
export const _$E = (flags, prefix, suffix, props, style, children, directives) => ({
    flags,
    prefix,
    suffix,
    props,
    style,
    children,
    directives,
});
export const _$P = (prefix, flags, i) => ({
    prefix,
    flags,
    i,
});
export const _$S = (stat, dyn) => ({
    stat,
    dyn,
});
export const _$t = (d, p) => ({
    d,
    p,
});
export const renderToString = (v) => {
    let result;
    const ctx = RENDER_CONTEXT;
    try {
        renderNode(v);
        result = ctx.t;
    }
    finally {
        ctx.t = "";
        ctx.s = 0;
        ctx.c.length = 0;
    }
    return result;
};
const renderNode = (v) => {
    if (v === false || v == null) {
        return;
    }
    if (typeof v === "object") {
        if (_isArray(v)) {
            for (let i = 0; i < v.length; i++) {
                renderNode(v[i]);
            }
        }
        else {
            const { d, p } = v;
            const { t, s } = d;
            if (t === 0 /* VNodeType.Template */) {
                const roots = s;
                for (let i = 0; i < roots.length; i++) {
                    const child = roots[i];
                    if (typeof child === "object") { // Element
                        renderTElement(p, child);
                    }
                    else if (typeof child === "string") { // Text
                        renderText(child);
                    }
                    else { // Expr
                        renderNode(p[child]);
                    }
                }
            }
            else if (t === 1 /* VNodeType.Component */) {
                renderNode(s(p)(p));
            }
            else { // Context
                const ctxStack = RENDER_CONTEXT.c;
                ctxStack.push(v);
                renderNode(p.c);
                ctxStack.pop();
            }
        }
    }
    else { // text
        if (v !== "") {
            renderText(typeof v === "number"
                ? "" + v
                : escapeHTMLText(v));
        }
    }
};
const renderText = (text) => {
    const ctx = RENDER_CONTEXT;
    if (ctx.s & 16777216 /* RenderState.PrevText */) {
        ctx.t += "<!>";
    }
    ctx.s = (ctx.s + 1) | 16777216 /* RenderState.PrevText */;
    ctx.t += text;
};
const renderTElement = (exprs, e) => {
    const ctx = RENDER_CONTEXT;
    const flags = e.flags;
    const offsets = (flags & 1 /* TFlags.GenerateOffsets */)
        ? []
        : void 0;
    const suffix = e.suffix;
    const props = e.props;
    const style = e.style;
    const directives = e.directives;
    let children = e.children;
    let openElement = e.prefix;
    if (props !== null) {
        for (let i = 0; i < props.length; i++) {
            const prop = props[i];
            const value = exprs[prop.i];
            if (value != null) {
                if (typeof value === "string") {
                    if (value === "") {
                        if (!(prop.flags & 1 /* TFlags.IgnoreEmptyString */)) {
                            openElement += prop.prefix;
                        }
                    }
                    else {
                        openElement += prop.prefix + '="' + escapeHTMLAttribute(value) + '"';
                    }
                }
                else if (typeof value === "boolean") {
                    if (value === true) {
                        openElement += prop.prefix;
                    }
                }
                else {
                    openElement += prop.prefix + '="' + value + '"';
                }
            }
        }
    }
    if (style !== null) {
        const dyn = style.dyn;
        let s = style.stat;
        if (dyn !== null) {
            for (let i = 0; i < dyn.length; i++) {
                const prop = dyn[i];
                const value = exprs[prop.i];
                if (value != null) {
                    if (typeof value === "string") {
                        if (value !== "") {
                            s += prop.prefix + escapeHTMLAttribute(value) + ';';
                        }
                    }
                    else if (value !== false) {
                        s += prop.prefix + value + ';';
                    }
                }
            }
        }
        if (s !== "") {
            openElement += ' style="' + s + '"';
        }
    }
    if (directives !== null) {
        for (let i = 0; i < directives.length; i++) {
            const v = exprs[directives[i]]();
            if (v !== void 0) {
                if (typeof v === "string") {
                    openElement += " " + v;
                }
                else {
                    const attrs = v.a;
                    const child = v.c;
                    if (attrs !== void 0) {
                        openElement += " " + attrs;
                    }
                    if (child !== void 0) {
                        children = child;
                    }
                }
            }
        }
    }
    ctx.s = (ctx.s + 1) & 16777215 /* RenderState.OffsetMask */;
    if (children === null) {
        ctx.t += openElement + ">" + suffix;
        return;
    }
    if (typeof children === "object") {
        let pushOffset = false;
        let offset = 0;
        const prevT = ctx.t;
        const prevS = ctx.s;
        ctx.t = "";
        ctx.s = 0;
        for (let i = 0; i < children.length; i++) {
            const child = children[i];
            if (typeof child === "object") { // Element
                if (pushOffset) {
                    pushOffset = false;
                    offsets.push(offset.toString(16));
                    offset = 0;
                }
                renderTElement(exprs, child);
            }
            else if (typeof child === "string") { // Text
                if (pushOffset) {
                    pushOffset = false;
                    offsets.push(offset.toString(16));
                    offset = 0;
                }
                renderText(child);
            }
            else { // Expr
                const prevS = ctx.s;
                ctx.s &= 16777216 /* RenderState.PrevText */;
                renderNode(exprs[child]);
                if (offsets !== void 0) {
                    offset += ctx.s & 16777215 /* RenderState.OffsetMask */;
                    pushOffset = true;
                }
                ctx.s = (prevS & 16777215 /* RenderState.OffsetMask */) | (ctx.s & 16777216 /* RenderState.PrevText */);
            }
        }
        if (offsets !== void 0) {
            openElement += ` &="${offsets.join(" ")}">`;
        }
        else {
            openElement += `>`;
        }
        ctx.t = prevT + openElement + ctx.t + suffix;
        ctx.s = prevS;
    }
    else if (typeof children === "number") { // textContent / innerHTML / <textarea .value={} />
        const v = exprs[children];
        ctx.t += openElement + ">";
        if (v != null) {
            if (typeof v === "string") {
                ctx.t += (flags & 2 /* TFlags.EscapeInnerHTML */)
                    ? escapeHTMLText(v)
                    : v;
            }
            else if (v !== false) {
                ctx.t += v;
            }
        }
        ctx.t += suffix;
    }
    else { // element directive
        ctx.t += openElement + ">" + children + suffix;
    }
};
const RENDER_CONTEXT = Object.seal({
    t: "",
    s: 0,
    c: [],
});
const _Array = Array;
const _isArray = _Array.isArray;
export const invalidate = () => {
    throw Error("invalidate function isn't available during Server-Side Rendering.");
};
export const useUnmount = () => {
    throw Error("useUnmount function isn't available during Server-Side Rendering.");
};
export const createRoot = () => {
    throw Error("createRoot function isn't available during Server-Side Rendering.");
};
export const dirtyCheck = () => {
    throw Error("dirtyCheck function isn't available during Server-Side Rendering.");
};
export const update = () => {
    throw Error("update function isn't available during Server-Side Rendering.");
};
export const unmount = () => {
    throw Error("unmount function isn't available during Server-Side Rendering.");
};
export const hydrate = () => {
    throw Error("hydrate function isn't available during Server-Side Rendering.");
};
//# sourceMappingURL=core.js.map